(ns fluree.db.auth
  (:require [fluree.db.dbproto :as dbproto]
            [fluree.db.util.async :refer [<? go-try]]))


(defn roles
  "Given an _auth identity, returns associated roles.
  First returns roles directly associated with auth ident.
  If none exist, attempts to retrieve default roles for associated
  user, if they exist."
  [db auth_id]
  (go-try
    (if (= 0 auth_id)                                       ;; root user, special case
      []
      ;; Lookup both auth roles and user roles in parallel
      (let [auth-roles-ch (dbproto/-query db {:select "?roles"
                                              :where  [[auth_id "_auth/roles" "?roles"]]})
            user-roles-ch (dbproto/-query db {:select "?roles"
                                              :where  [["?user" "_user/auth" auth_id]
                                                       ["?user" "_user/roles" "?roles"]]})]
        (if-let [auth-roles (not-empty (<? auth-roles-ch))]
          auth-roles
          (<? user-roles-ch))))))
